#library(MCMCpack)
#source("sub_functions.R")

## HL test ##

hosmerlem = function(y, yhat, g=10) {
  cutyhat = cut(yhat,
     breaks = quantile(yhat, probs=seq(0,1, 1/g)), include.lowest=TRUE)
  obs = xtabs(cbind(1 - y, y) ~ cutyhat)
  expect = xtabs(cbind(1 - yhat, yhat) ~ cutyhat)
  chisq = sum((obs - expect)^2/expect)
  P = 1 - pchisq(chisq, g - 2)
  return(list(chisq=chisq,p.value=P))
}






## overall ##


test_statistic_for_given_threshold<-function(threshold,response_resid,risk){

n<-length(response_resid)

element<-sum(response_resid[risk<=threshold[1]]^2)+sum(response_resid[risk>=threshold[2]]^2)

nc<-sum(risk<=threshold[1])+sum(risk>=threshold[2])

z<-(element-nc)/sqrt(n)

return(z)


}


test_statistic_for_all_samples<-function(response_resid,risk){

n<-length(response_resid)

element<-sum(response_resid^2)

z<-(element-n)/sqrt(n)

return(z)


}


pairwise_covariance<-function(index,cutoff,nu_c,risk,fitted_value,omega){


n<-length(risk)

cutoff_vec<-c(min(cutoff[index[1],1],cutoff[index[2],1]),max(cutoff[index[1],2],cutoff[index[2],2]))

nu<-rbind(nu_c[index[1],],nu_c[index[2],])

sigma_square<-sum((1-2*fitted_value[risk<=cutoff_vec[1]|risk>=cutoff_vec[2]])^2/(fitted_value[risk<=cutoff_vec[1]|risk>=cutoff_vec[2]]*(1-fitted_value[risk<=cutoff_vec[1]|risk>=cutoff_vec[2]])))/n-t(nu[1,])%*%solve(omega)%*%nu[2,]
  
return(sigma_square)


}

## additive without covariates###

negative_log_lik_additive<-function(beta,snp,disease){


exp_gamma_x_general<-NULL
for(i in 1:dim(snp)[2]){
exp_gamma_x_general<-cbind(exp_gamma_x_general,snp[,i]*(beta[i+1]))
}

s<-apply(snp,1,sum)

lik<-sum(disease*(beta[1]+log(apply(exp_gamma_x_general,1,sum)-(s-1)))-log(1+exp(beta[1]+log(apply(exp_gamma_x_general,1,sum)-(s-1)))))

return(-1*lik)



}


score_additive<-function(beta,snp,disease){


s<-apply(snp,1,sum)
p<-dim(snp)[2]+1
exp_gamma_x_general<-NULL
for(i in 1:dim(snp)[2]){
exp_gamma_x_general<-cbind(exp_gamma_x_general,snp[,i]*(beta[i+1]))
}

exp_gamma_x<-cbind(exp_gamma_x_general,(apply(exp_gamma_x_general,1,sum))-(s-1))

X_general<-NULL

for(i in 1:dim(snp)[2]){

X_general<-cbind(X_general,exp_gamma_x[,i]/(beta[i+1]*exp_gamma_x[,dim(exp_gamma_x)[2]]))

}

linear_general<-exp(beta[1]+log(exp_gamma_x[,p]))/(1+exp(beta[1]+log(exp_gamma_x[,p])))

score<-t(cbind(disease-linear_general,(disease-linear_general)*X_general))

score<-apply(score,1,sum)

return(-1*score)

}


expected_i_additive<-function(gamma,x){

dim_x<-dim(x)[2]

s<-apply(x,1,sum)




exp_gamma_x_general<-NULL
for(i in 1:dim_x){
exp_gamma_x_general<-cbind(exp_gamma_x_general,x[,i]*(gamma[i+1]))
}

linear_predictor<-apply(exp_gamma_x_general,1,sum)-(s-1)


linear_s<-gamma[1]+log(linear_predictor)

pi<-exp(linear_s)/(1+exp(linear_s))

common<--1*x/(linear_predictor)

variance<-pi*(1-pi)

index<-t(combn(1:dim(x)[2],2))

gamma_k_gamma_j<-NULL
for(j in 1:dim(index)[1]){
gamma_k_gamma_j<-c(gamma_k_gamma_j,-1*sum(common[,index[j,1]]*common[,index[j,2]]*variance))
}


gamma_j_gamma_j<-NULL
for(j in 1:dim(x)[2]){

gamma_j_gamma_j<-c(gamma_j_gamma_j,-1*sum(common[,j]^2*variance))

}

first<-c(-1*sum(variance), apply(common*variance,2,sum))

rest<-matrix(rep(0,dim(x)[2]*dim(x)[2]),nrow=dim(x)[2])

rest[lower.tri(rest)]<-gamma_k_gamma_j

rest<-t(rest)+rest
diag(rest)<-gamma_j_gamma_j


expected_v<-rbind(first,cbind(first[-1],rest))

return(expected_v)

}

## additive with covariates ##


fr_covariate_negative<-function(beta,x_all,disease,num_of_covariate){
snp<-x_all[,1:(num_of_covariate)]
covariate<-x_all[,-(1:(num_of_covariate))]
exp_gamma_x_general<-NULL
for(i in 1:dim(snp)[2]){
exp_gamma_x_general<-cbind(exp_gamma_x_general,(snp[,i]*beta[i+1]))
}

s<-apply(snp,1,sum)

lik<-sum(disease*(beta[1]+log(apply(exp_gamma_x_general,1,sum)-(s-1))+beta[-(1:(num_of_covariate+1))]%*%t(covariate))-log(1+exp(beta[1]+log(apply(exp_gamma_x_general,1,sum)-(s-1))+beta[-(1:(num_of_covariate+1))]%*%t(covariate))))


#lik<-sum(disease*(beta[1]+log(1+apply(exp_gamma_x_general,1,sum)-dim(snp)[2])+beta[-(1:(num_of_covariate+1))]%*%t(covariate))-log(1+exp(beta[1]+log(1+apply(exp_gamma_x_general,1,sum)-dim(snp)[2])+beta[-(1:(num_of_covariate+1))]%*%t(covariate))))

return(lik*-1)

}


score_covariate_negative<-function(beta,x_all,disease,num_of_covariate){

snp<-x_all[,1:(num_of_covariate)]
covariate<-x_all[,-(1:(num_of_covariate))]

p<-dim(snp)[2]+1
exp_gamma_x_general<-NULL
for(i in 1:dim(snp)[2]){
exp_gamma_x_general<-cbind(exp_gamma_x_general,(snp[,i]*beta[i+1]))
}

s<-apply(snp,1,sum)

exp_gamma_x<-cbind(exp_gamma_x_general,(apply(exp_gamma_x_general,1,sum))-(s-1))

X_general<-NULL

for(i in 1:dim(snp)[2]){

X_general<-cbind(X_general,exp_gamma_x[,i]/(beta[i+1]*exp_gamma_x[,dim(exp_gamma_x)[2]]))

}

linear_general<-exp(beta[1]+log(exp_gamma_x[,p])+beta[-(1:(num_of_covariate+1))]%*%t(covariate))/(1+exp(beta[1]+log(exp_gamma_x[,p])+beta[-(1:(num_of_covariate+1))]%*%t(covariate)))

linear_general<-as.vector(linear_general)

score<-t(cbind(disease-linear_general,(disease-linear_general)*X_general,(disease-linear_general)*covariate))

score<-apply(score,1,sum)

return(score*-1)
}



expected_i_covariate<-function(combined_beta_gamma,x_all,num_of_covariate){

beta<-combined_beta_gamma[1:(num_of_covariate+1)]

gamma<-combined_beta_gamma[-(1:(num_of_covariate+1))]

x<-x_all[,1:num_of_covariate]

z<-x_all[,-(1:num_of_covariate)]
dim_x<-dim(x)[2]

s<-apply(x,1,sum)


exp_gamma_x_general<-NULL
for(i in 1:dim_x){
exp_gamma_x_general<-cbind(exp_gamma_x_general,x[,i]*(beta[i+1]))
}

linear_s<-beta[1]+log(apply(exp_gamma_x_general,1,sum)-(s-1))+z%*%gamma

pi<-exp(linear_s)/(1+exp(linear_s))

common<--1*x/(apply(exp_gamma_x_general,1,sum)-(s-1))

variance<-pi*(1-pi)

index<-t(combn(1:dim(x)[2],2))

beta_k_beta_j<-NULL
for(j in 1:dim(index)[1]){
beta_k_beta_j<-c(beta_k_beta_j,-1*sum(common[,index[j,1]]*common[,index[j,2]]*variance))
}


beta_j_beta_j<-NULL
for(j in 1:dim(x)[2]){

beta_j_beta_j<-c(beta_j_beta_j,-1*sum(common[,j]^2*variance))

}

#first<-c(-1*sum(variance), apply(common*variance,2,sum))

first<-c(-1*sum(variance),t(common)%*%variance)

rest<-matrix(rep(0,dim(x)[2]*dim(x)[2]),nrow=dim(x)[2])

rest[lower.tri(rest)]<-beta_k_beta_j

rest<-t(rest)+rest
diag(rest)<-beta_j_beta_j


expected_v<-rbind(first,cbind(first[-1],rest))


index_g<-t(combn(1:dim(z)[2],2))

gamma_k_gamma_j<-NULL
for(j in 1:dim(index_g)[1]){
gamma_k_gamma_j<-c(gamma_k_gamma_j,-1*sum(z[,index_g[j,1]]*z[,index_g[j,2]]*variance))

}


#gamma_j_gamma_j<--1*apply(z^2*variance,2,sum)
gamma_j_gamma_j<--1*t(z^2)%*%variance


rest<-matrix(rep(0,dim(z)[2]*dim(z)[2]),nrow=dim(z)[2])

rest[lower.tri(rest)]<-gamma_k_gamma_j

rest<-t(rest)+rest

diag(rest)<-gamma_j_gamma_j

beta_gamma<-NULL


for(j in 1:dim(exp_gamma_x_general)[2]){
for(i in 1:dim(z)[2]){
#beta_gamma<-c(beta_gamma,-1*sum(z[,i]*variance*exp_gamma_x_general[,j]/(apply(exp_gamma_x_general,1,sum)-(s-1))))
beta_gamma<-c(beta_gamma,-1*sum(z[,i]*variance*x[,j]/(apply(exp_gamma_x_general,1,sum)-(s-1))))


}
}

beta_gamma<-matrix(beta_gamma,byrow=T,ncol=dim(z)[2])

gamma_alpha<--1*t(variance)%*%z


return(rbind(cbind(expected_v,rbind(gamma_alpha,beta_gamma)),cbind(t(rbind(gamma_alpha,beta_gamma)),rest)))
}




GOFR<-function(formula,data, link="logit",grid=c(0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9), n_simulations, tol=10^-5, covariate=NULL){

  if (missing(data)) 
        data <- environment(formula)

 if(link=="logit") {

if(is.null(covariate)==T){
logit_link(formula,data, grid, n_simulations)
}else{logit_link_covariate(formula,data,grid,n_simulations,covariate)
}

}


 
else if(link=="identity"){

if(is.null(covariate)==T){
identity_link(formula,data,grid,n_simulations, tol)
}

else{identity_link_covariate(formula,data,grid,n_simulations, tol,covariate)
 }

}



else{
if(is.null(covariate)==T){

 logit_link(formula,data,grid,n_simulations)
}
else {logit_link_covariate(formula,data,grid,n_simulations,covariate)
}

}
 
 }

getSummary<-function(fit){

clss<-class(fit)

cols <- c("Estimate", "Std.Error", "Z.value", "Pvalue")
 
 sided <- 2
       
fit_coefficients<-cbind( fit$coefficients, sqrt(diag(fit$cov)), fit$coefficients/sqrt(diag(fit$cov)))

 fit_coefficients <- cbind(fit_coefficients,sided * pnorm(abs(as.numeric(fit_coefficients[, 3])), lower.tail = FALSE))

fit_coefficients<-as.data.frame(fit_coefficients)

names(fit_coefficients)<-c("Estimate","Std.Error","z value", "Pr(>|z|)")

fit_test_statistic<-c(fit$TBM_ts, fit$HL_ts, fit$Windmeijer_ts)

names(fit_test_statistic)<-c("Tail-Based Max Test","Hosmer and Lemeshow Test", "Windmeijer Test")

test_result<-cbind(fit_test_statistic,c(fit$TBM_pv, fit$HL_pv, fit$Windmeijer_pv))

test_result<-as.data.frame(test_result)

names(test_result)<-c("Test Statistic"," P-value")

return_value<-list(summary_of_fit=fit_coefficients, summary_of_test=test_result)

return(return_value)

}

identity_link<-function(formula, data, grid, n_simulations, tol){

  call <- match.call()
        
if (missing(data)) 
        data <- environment(formula)

    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())
    
    mt <- attr(mf, "terms")
    Y <- model.response(mf, "any")
    if (length(dim(Y)) == 1L) {
        nm <- rownames(Y)
        dim(Y) <- NULL
        if (!is.null(nm)) 
            names(Y) <- nm
    }
    X <- if (!is.empty.model(mt)) 
        model.matrix(mt, mf, contrasts)
    else matrix(, NROW(Y), 0L)

 x<-(X[,-1])

n<-length(Y)

if(missing(data)) glm_fit<-glm(formula,family=binomial(link=logit))

if(!missing(data)) (glm_fit<-glm(formula,data=data, family=binomial(link=logit)))

beta<-glm_fit$coefficients

beta_slope<-beta[-1]

beta_slope[which(beta_slope<=0)]<-0.01

beta<-c(beta[1],exp(beta_slope))

optim_result<-constrOptim(theta=beta,f=negative_log_lik_additive, grad=score_additive,ui=rbind(rep(0,dim(x)[2]+1),cbind(rep(0,dim(x)[2]),diag(1,nrow=dim(x)[2]))),ci=c(-1,rep(1,dim(x)[2])), method="BFGS",snp=x,disease=Y)




for(j in 1:50){

optim_result_previous<-optim_result

optim_result<-constrOptim(theta=optim_result_previous$par,f=negative_log_lik_additive,grad=score_additive,ui=rbind(rep(0,dim(x)[2]+1),cbind(rep(0,dim(x)[2]),diag(1,nrow=dim(x)[2]))),ci=c(-1,rep(1,dim(x)[2])),method="BFGS",snp=x,disease=Y)


#if(abs(optim_result_previous$value-optim_result$value)<10^-5& sqrt(sum((optim_result$par-optim_result_previous$par)^2))<10^-5)break


if(abs(optim_result_previous$value-optim_result$value)<tol& sqrt(sum((optim_result$par-optim_result_previous$par)^2))<tol)break



}


if(j==50) print("warning: the iteration limit has been reached")

exp_gamma_x_general<-NULL

for(i in 1:dim(x)[2]){

exp_gamma_x_general<-cbind(exp_gamma_x_general,x[,i]*optim_result$par[i+1])

}

s<-apply(x,1,sum)


m_hat<-optim_result$par[1]+log(apply(exp_gamma_x_general,1,sum)-(s-1))




fitted_value<-exp(m_hat)/(1+exp(m_hat))

risk<-log(apply(exp_gamma_x_general,1,sum)-(s-1))

#threshold<-quantile(risk,probs=seq(0,1,1/ngrid),include.lowest=TRUE)

ngrid<-length(grid)+1

threshold<-quantile(risk,probs=c(0,grid,1),include.lowest=TRUE)

bin_location<-names(threshold)

if(ngrid%%2==0){

median_location<-which(bin_location=="50%")

threshold_pair1<-cbind(rep(threshold[1]-1,length(threshold)-1-median_location),threshold[(length(threshold)-1):(median_location+1)])

threshold_pair2<-cbind(rep(threshold[2:(median_location-1)],each=length(threshold)-median_location),rep(c(threshold[length(threshold)]+1,threshold[(length(threshold)-1):(median_location+1)]),median_location-2))

threshold_pair<-rbind(threshold_pair1,threshold_pair2,rep(threshold[median_location],2))

}


if(ngrid%%2!=0){

median_location<-length(threshold)/2

threshold_pair1<-cbind(rep(threshold[1]-1,length(threshold)-1-median_location),threshold[(length(threshold)-1):(median_location+1)])

threshold_pair2<-cbind(rep(threshold[2:(median_location)],each=length(threshold)-median_location),rep(c(threshold[length(threshold)]+1,threshold[(length(threshold)-1):(median_location+1)]),median_location-1))

threshold_pair<-rbind(threshold_pair1,threshold_pair2,rep(median(risk),2))

}

response_resid<-(Y-fitted_value)/sqrt(fitted_value*(1-fitted_value))

test_statistic<-c(apply(threshold_pair[-dim(threshold_pair)[1],],1,test_statistic_for_given_threshold,response_resid, risk),test_statistic_for_all_samples(response_resid,risk))

f<-fitted_value*(1-fitted_value)

X_general<-NULL

for(i in 1:dim(x)[2]){

X_general<-cbind(X_general,x[,i]*f/(apply(exp_gamma_x_general,1,sum)-(s-1)))

}

fx<-cbind(f,X_general)

nu<-(1-2*fitted_value)/(fitted_value*(1-fitted_value))*fx
 
nu<-apply(nu,2,mean)

omega<-expected_i_additive(optim_result$par,x)/n*-1

 each_z<-NULL
 sigma<-NULL
 mu<-NULL
nu_c<-NULL
 for(j in 1:dim(threshold_pair)[1]){
 
if(j==dim(threshold_pair)[1]){ 
   
element<-sum(response_resid^2)

f<-fitted_value*(1-fitted_value)

fx<-cbind(f,X_general)

 nu<-(1-2*fitted_value)/(fitted_value*(1-fitted_value))*fx
 
 nu<-apply(nu,2,sum)/n
 
nu_c<-rbind(nu_c,nu)

sigma_square<-mean((1-2*fitted_value)^2/(fitted_value*(1-fitted_value)))-t(nu)%*%solve(omega)%*%nu

 sigma<-c(sigma,sigma_square)

}

if(j!=dim(threshold_pair)[1]){

nc<-sum(risk>=threshold_pair[j,2])+sum(risk<=threshold_pair[j,1])

element<-sum(response_resid[risk>=threshold_pair[j,2]]^2)+sum(response_resid[risk<=threshold_pair[j,1]]^2)
 
 
 f<-fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]]*(1-fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]])
 
fx<-cbind(f,X_general[(risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]),])


 nu<-(1-2*fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]])/f*fx

 nu<-apply(nu,2,sum)/n
 
nu_c<-rbind(nu_c,nu)

  
 sigma_square<-sum((1-2*fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]])^2/(fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]]*(1-fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]])))/n-t(nu)%*%solve(omega)%*%nu
  
 sigma<-c(sigma,sigma_square)

 }
 }


pairwise<-apply(t(combn(1:dim(threshold_pair)[1],2)),1,pairwise_covariance,threshold_pair,nu_c,risk,fitted_value,omega)


 csigma<-c(sigma,pairwise)

no_order<-cbind(rbind(matrix(rep(1:dim(threshold_pair)[1],2),ncol=2),t(combn(1:dim(threshold_pair)[1],2))),csigma )


intermediate_sigma<-no_order[order(no_order[,1],no_order[,2]),]

Sigma<-xpnd(intermediate_sigma[,3],dim(threshold_pair)[1])

Sigma<-cov2cor(Sigma)

 
 simulated_sample<-mvrnorm(n=n_simulations,rep(0,dim(threshold_pair)[1]),Sigma)
 
 empirical_max<-apply(abs(simulated_sample),1,max)
 
marginal_test_stat<-test_statistic/sqrt(csigma[1:length(test_statistic)])


current_max<-max(abs(marginal_test_stat))
 
 marginal_p<-NULL

for(j in 1:length(marginal_test_stat)){

marginal_p<-c(marginal_p,2*(1-pnorm(abs(marginal_test_stat[j]))))
}

pval<-length(empirical_max[empirical_max>=current_max])/length(empirical_max)

hl<-hosmerlem(Y,fitted_value,g=10)


retval<-list(coefficients=c(optim_result$par[1],optim_result$par[-1]-1),fitted.values=fitted_value, cov=solve(omega*n), TBM_ts=current_max, TBM_pv=pval, Windmeijer_ts=abs(marginal_test_stat[length(marginal_test_stat)]), Windmeijer_pv=marginal_p[length(marginal_p)], HL_ts=hl$chisq, HL_pv=hl$p.value)





return(retval)

}




logit_link<-function(formula,  data, grid, n_simulations){

    call <- match.call()
       

if (missing(data)) 
        data <- environment(formula)


    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())
    
    mt <- attr(mf, "terms")
    Y <- model.response(mf, "any")
    if (length(dim(Y)) == 1L) {
        nm <- rownames(Y)
        dim(Y) <- NULL
        if (!is.null(nm)) 
            names(Y) <- nm
    }
    X <- if (!is.empty.model(mt)) 
        model.matrix(mt, mf, contrasts)
    else matrix(, NROW(Y), 0L)

 x<-(X[,-1])

n<-length(Y)

if(missing(data)) glm_fit<-glm(formula,family=binomial(link=logit))

if(!missing(data)) (glm_fit<-glm(formula,data=data, family=binomial(link=logit)))

fitted_value<-fitted(glm_fit)

risk<-x%*%glm_fit$coefficients[-1]


ngrid<-length(grid)+1

threshold<-quantile(risk,probs=c(0,grid,1),include.lowest=TRUE)




#threshold<-quantile(risk,probs=seq(0,1,1/ngrid),include.lowest=TRUE)

bin_location<-names(threshold)

if(ngrid%%2==0){

median_location<-which(bin_location=="50%")

threshold_pair1<-cbind(rep(threshold[1]-1,length(threshold)-1-median_location),threshold[(length(threshold)-1):(median_location+1)])

threshold_pair2<-cbind(rep(threshold[2:(median_location-1)],each=length(threshold)-median_location),rep(c(threshold[length(threshold)]+1,threshold[(length(threshold)-1):(median_location+1)]),median_location-2))

threshold_pair<-rbind(threshold_pair1,threshold_pair2,rep(threshold[median_location],2))

}

if(ngrid%%2!=0){

median_location<-length(threshold)/2

threshold_pair1<-cbind(rep(threshold[1]-1,length(threshold)-1-median_location),threshold[(length(threshold)-1):(median_location+1)])

threshold_pair2<-cbind(rep(threshold[2:(median_location)],each=length(threshold)-median_location),rep(c(threshold[length(threshold)]+1,threshold[(length(threshold)-1):(median_location+1)]),median_location-1))

threshold_pair<-rbind(threshold_pair1,threshold_pair2,rep(median(risk),2))

}

response_resid<-resid(glm_fit,type="pearson")

test_statistic<-c(apply(threshold_pair[-dim(threshold_pair)[1],],1,test_statistic_for_given_threshold,response_resid, risk),test_statistic_for_all_samples(response_resid,risk))

each_z<-NULL
sigma<-NULL
mu<-NULL
nu_c<-NULL

for(j in 1:dim(threshold_pair)[1]){
 
if(j==dim(threshold_pair)[1]){ 
    
element<-sum(response_resid^2)

f<-fitted_value*(1-fitted_value)
 
nu<-(1-2*fitted_value)*f/(fitted_value*(1-fitted_value))*X
 
nu<-apply(nu,2,sum)/n
 
nu_c<-rbind(nu_c,nu)

coeff_cov<-vcov(glm_fit)
 
omega<-solve(coeff_cov*n)
 
sigma_square<-sum((1-2*fitted_value)^2/(fitted_value*(1-fitted_value)))/n-t(nu)%*%solve(omega)%*%nu

sigma<-c(sigma,sigma_square)

}

if(j!=dim(threshold_pair)[1]){

nc<-sum(risk>=threshold_pair[j,2])+sum(risk<=threshold_pair[j,1])

element<-sum(response_resid[risk>=threshold_pair[j,2]]^2)+sum(response_resid[risk<=threshold_pair[j,1]]^2)
 
 
f<-fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]]*(1-fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]])
 
x_matrix<-cbind(rep(1,nc),x[(risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]),])
 
nu<-(1-2*fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]])*f/(fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]]*(1-fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]]))*x_matrix
 
nu<-apply(nu,2,sum)/n
 
nu_c<-rbind(nu_c,nu)

coeff_cov<-vcov(glm_fit)
 
omega<-solve(coeff_cov*n)
 
sigma_square<-sum((1-2*fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]])^2/(fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]]*(1-fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]])))/n-t(nu)%*%solve(omega)%*%nu
  
sigma<-c(sigma,sigma_square)

 }
 }

pairwise<-apply(t(combn(1:dim(threshold_pair)[1],2)),1,pairwise_covariance,threshold_pair,nu_c,risk,fitted_value,omega)

 csigma<-c(sigma,pairwise)

no_order<-cbind(rbind(matrix(rep(1:dim(threshold_pair)[1],2),ncol=2),t(combn(1:dim(threshold_pair)[1],2))),csigma )

intermediate_sigma<-no_order[order(no_order[,1],no_order[,2]),]

Sigma<-xpnd(intermediate_sigma[,3],dim(threshold_pair)[1])

Sigma<-cov2cor(Sigma)

 simulated_sample<-mvrnorm(n=n_simulations,rep(0,dim(threshold_pair)[1]),Sigma)
 
 empirical_max<-apply(abs(simulated_sample),1,max)
 
marginal_test_stat<-test_statistic/sqrt(csigma[1:length(test_statistic)])

current_max<-max(abs(marginal_test_stat))
 
marginal_p<-NULL

for(j in 1:length(test_statistic)){

marginal_p<-c(marginal_p,2*(1-pnorm(abs(marginal_test_stat[j]))))
}

pval<-length(empirical_max[empirical_max>=current_max])/length(empirical_max)

hl<-hosmerlem(Y,fitted_value,g=10)

#retval<-list(threshold_pair=threshold_pair,coefficients=glm_fit$coefficients,fitted.values=fitted_value, cov=solve(omega*n), test_statistic=current_max, p.value=pval, windmeijer_test_statistic=marginal_test_stat[length(marginal_test_stat)], windmeijer_p.value=marginal_p[length(marginal_p)], hl_test_statistic=hl$chisq, hl_p.value=hl$p.value)


retval<-list(coefficients=glm_fit$coefficients,fitted.values=fitted_value, cov=solve(omega*n), TBM_ts=current_max, TBM_pv=pval, Windmeijer_ts=abs(marginal_test_stat[length(marginal_test_stat)]), Windmeijer_pv=marginal_p[length(marginal_p)], HL_ts=hl$chisq, HL_pv=hl$p.value)





return(retval)

}




logit_link_covariate<-function(formula,  data, grid, n_simulations,covariate){

    call <- match.call()
       

if (missing(data)) 
        data <- environment(formula)


    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())
    
    mt <- attr(mf, "terms")
    Y <- model.response(mf, "any")
    if (length(dim(Y)) == 1L) {
        nm <- rownames(Y)
        dim(Y) <- NULL
        if (!is.null(nm)) 
            names(Y) <- nm
    }
    X <- if (!is.empty.model(mt)) 
        model.matrix(mt, mf, contrasts)
    else matrix(, NROW(Y), 0L)


#print(X)
dim_X<-dim(X)[2]
dim_covariate<-length(covariate)



 x<-(X[,-c(1,(dim_X-dim_covariate+1):dim_X)])

X<-X[,-1]

n<-length(Y)

if(missing(data)) glm_fit<-glm(formula,family=binomial(link=logit))

if(!missing(data)) (glm_fit<-glm(formula,data=data, family=binomial(link=logit)))

fitted_value<-fitted(glm_fit)

risk<-x%*%glm_fit$coefficients[2:(dim_X-dim_covariate)]


ngrid<-length(grid)+1

threshold<-quantile(risk,probs=c(0,grid,1),include.lowest=TRUE)





bin_location<-names(threshold)

if(ngrid%%2==0){

median_location<-which(bin_location=="50%")

threshold_pair1<-cbind(rep(threshold[1]-1,length(threshold)-1-median_location),threshold[(length(threshold)-1):(median_location+1)])

threshold_pair2<-cbind(rep(threshold[2:(median_location-1)],each=length(threshold)-median_location),rep(c(threshold[length(threshold)]+1,threshold[(length(threshold)-1):(median_location+1)]),median_location-2))

threshold_pair<-rbind(threshold_pair1,threshold_pair2,rep(threshold[median_location],2))

}

if(ngrid%%2!=0){

median_location<-length(threshold)/2

threshold_pair1<-cbind(rep(threshold[1]-1,length(threshold)-1-median_location),threshold[(length(threshold)-1):(median_location+1)])

threshold_pair2<-cbind(rep(threshold[2:(median_location)],each=length(threshold)-median_location),rep(c(threshold[length(threshold)]+1,threshold[(length(threshold)-1):(median_location+1)]),median_location-1))

threshold_pair<-rbind(threshold_pair1,threshold_pair2,rep(median(risk),2))

}

num_of_cutoff<-dim(threshold_pair)[1]







response_resid<-resid(glm_fit,type="pearson")

test_statistic<-c(apply(threshold_pair[-dim(threshold_pair)[1],],1,test_statistic_for_given_threshold,response_resid, risk),test_statistic_for_all_samples(response_resid,risk))

each_z<-NULL
sigma<-NULL
mu<-NULL
nu_c<-NULL

for(j in 1:dim(threshold_pair)[1]){
 
if(j==dim(threshold_pair)[1]){ 
    
element<-sum(response_resid^2)

f<-fitted_value*(1-fitted_value)
  x_matrix<-cbind(rep(1,n),X)

nu<-(1-2*fitted_value)*f/(fitted_value*(1-fitted_value))*x_matrix
 
nu<-apply(nu,2,sum)/n
 
nu_c<-rbind(nu_c,nu)

coeff_cov<-vcov(glm_fit)
 
omega<-solve(coeff_cov*n)
 
sigma_square<-sum((1-2*fitted_value)^2/(fitted_value*(1-fitted_value)))/n-t(nu)%*%solve(omega)%*%nu

sigma<-c(sigma,sigma_square)

}

if(j!=dim(threshold_pair)[1]){

nc<-sum(risk>=threshold_pair[j,2])+sum(risk<=threshold_pair[j,1])

element<-sum(response_resid[risk>=threshold_pair[j,2]]^2)+sum(response_resid[risk<=threshold_pair[j,1]]^2)
 
 
f<-fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]]*(1-fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]])
 
x_matrix<-cbind(rep(1,nc),X[(risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]),])
 
nu<-(1-2*fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]])*f/(fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]]*(1-fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]]))*x_matrix
 
nu<-apply(nu,2,sum)/n
 
nu_c<-rbind(nu_c,nu)

coeff_cov<-vcov(glm_fit)
 
omega<-solve(coeff_cov*n)
 
sigma_square<-sum((1-2*fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]])^2/(fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]]*(1-fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]])))/n-t(nu)%*%solve(omega)%*%nu
  
sigma<-c(sigma,sigma_square)

 }
 }

pairwise<-apply(t(combn(1:dim(threshold_pair)[1],2)),1,pairwise_covariance,threshold_pair,nu_c,risk,fitted_value,omega)

 csigma<-c(sigma,pairwise)

no_order<-cbind(rbind(matrix(rep(1:dim(threshold_pair)[1],2),ncol=2),t(combn(1:dim(threshold_pair)[1],2))),csigma )

intermediate_sigma<-no_order[order(no_order[,1],no_order[,2]),]

Sigma<-xpnd(intermediate_sigma[,3],dim(threshold_pair)[1])

Sigma<-cov2cor(Sigma)

 simulated_sample<-mvrnorm(n=n_simulations,rep(0,dim(threshold_pair)[1]),Sigma)
 
 empirical_max<-apply(abs(simulated_sample),1,max)
 
marginal_test_stat<-test_statistic/sqrt(csigma[1:length(test_statistic)])

current_max<-max(abs(marginal_test_stat))
 
marginal_p<-NULL

for(j in 1:length(test_statistic)){

marginal_p<-c(marginal_p,2*(1-pnorm(abs(marginal_test_stat[j]))))
}

pval<-length(empirical_max[empirical_max>=current_max])/length(empirical_max)

hl<-hosmerlem(Y,fitted_value,g=10)


retval<-list(coefficients=glm_fit$coefficients,fitted.values=fitted_value, cov=solve(omega*n), TBM_ts=current_max, TBM_pv=pval, Windmeijer_ts=abs(marginal_test_stat[length(marginal_test_stat)]), Windmeijer_pv=marginal_p[length(marginal_p)], HL_ts=hl$chisq, HL_pv=hl$p.value)






return(retval)

}
identity_link_covariate<-function(formula, data, grid, n_simulations, tol,covariate){

  call <- match.call()
        
if (missing(data)) 
        data <- environment(formula)

    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())
    
    mt <- attr(mf, "terms")
    Y <- model.response(mf, "any")
    if (length(dim(Y)) == 1L) {
        nm <- rownames(Y)
        dim(Y) <- NULL
        if (!is.null(nm)) 
            names(Y) <- nm
    }
    X <- if (!is.empty.model(mt)) 
        model.matrix(mt, mf, contrasts)
    else matrix(, NROW(Y), 0L)

dim_X<-dim(X)[2]

dim_covariate<-length(covariate)



x<-(X[,-c(1,(dim_X-dim_covariate+1):dim_X)])

dim_x<-dim(x)[2]

z<-X[,(dim_X-dim_covariate+1):dim_X]


X<-X[,-1]

s<-apply(x,1,sum)


n<-length(Y)

if(missing(data)) glm_fit<-glm(formula,family=binomial(link=logit))

if(!missing(data)) (glm_fit<-glm(formula,data=data, family=binomial(link=logit)))

beta<-glm_fit$coefficients


beta_slope<-beta[2:(dim_x+1)]

beta_slope[which(beta_slope<=0)]<-0.01

beta<-c(beta[1],exp(beta_slope),beta[-(1:(dim_x+1))])



ui=rbind(rbind(rep(0,dim(x)[2]+1+dim_covariate),cbind(rep(0,dim(x)[2]),diag(1,nrow=dim(x)[2]),cbind(matrix(rep(0,dim(x)[2]*(dim_covariate)),ncol=dim_covariate)))),matrix(rep(0,(dim_covariate)*(dim_x+dim_covariate+1)),nrow=dim_covariate))


optim_result<-constrOptim(beta,fr_covariate_negative,score_covariate_negative,ui=ui,ci=c(-1,rep(1,dim(x)[2]),rep(-1,dim_covariate)),method="BFGS",x=X,disease=Y,num_of_covariate=dim_x)







for(j in 1:50){

optim_result_previous<-optim_result

optim_result<-constrOptim(optim_result_previous$par,fr_covariate_negative,score_covariate_negative,ui=ui,ci=c(-1,rep(1,dim(x)[2]),rep(-1,dim_covariate)),method="BFGS",x=X,disease=Y,num_of_covariate=dim_x)




if(abs(optim_result_previous$value-optim_result$value)<tol& sqrt(sum((optim_result$par-optim_result_previous$par)^2))<tol)break



}


if(j==50) print("warning: the iteration limit has been reached")

exp_gamma_x_general<-NULL

for(i in 1:dim(x)[2]){

exp_gamma_x_general<-cbind(exp_gamma_x_general,x[,i]*optim_result$par[i+1])

}

s<-apply(x,1,sum)


risk<-log(apply(exp_gamma_x_general,1,sum)-(s-1))

m_hat<-optim_result$par[1]+risk+z%*%optim_result$par[-(1:(dim_x+1))]








fitted_value<-exp(m_hat)/(1+exp(m_hat))



ngrid<-length(grid)+1

threshold<-quantile(risk,probs=c(0,grid,1),include.lowest=TRUE)

bin_location<-names(threshold)

if(ngrid%%2==0){

median_location<-which(bin_location=="50%")

threshold_pair1<-cbind(rep(threshold[1]-1,length(threshold)-1-median_location),threshold[(length(threshold)-1):(median_location+1)])

threshold_pair2<-cbind(rep(threshold[2:(median_location-1)],each=length(threshold)-median_location),rep(c(threshold[length(threshold)]+1,threshold[(length(threshold)-1):(median_location+1)]),median_location-2))

threshold_pair<-rbind(threshold_pair1,threshold_pair2,rep(threshold[median_location],2))

}


if(ngrid%%2!=0){

median_location<-length(threshold)/2

threshold_pair1<-cbind(rep(threshold[1]-1,length(threshold)-1-median_location),threshold[(length(threshold)-1):(median_location+1)])

threshold_pair2<-cbind(rep(threshold[2:(median_location)],each=length(threshold)-median_location),rep(c(threshold[length(threshold)]+1,threshold[(length(threshold)-1):(median_location+1)]),median_location-1))

threshold_pair<-rbind(threshold_pair1,threshold_pair2,rep(median(risk),2))

}

response_resid<-(Y-fitted_value)/sqrt(fitted_value*(1-fitted_value))

test_statistic<-c(apply(threshold_pair[-dim(threshold_pair)[1],],1,test_statistic_for_given_threshold,response_resid, risk),test_statistic_for_all_samples(response_resid,risk))

f<-fitted_value*(1-fitted_value)

X_general<-NULL

for(i in 1:dim(x)[2]){

X_general<-cbind(X_general,x[,i]*f/(apply(exp_gamma_x_general,1,sum)-(s-1)))

}


for(i in 1:dim(z)[2]){
X_general<-cbind(X_general,z[,i]*f)

}




fx<-cbind(f,X_general)

 nu<-NULL
for(i in 1:dim(fx)[2]){

nu<-cbind(nu,fx[,i]*(1-2*fitted_value)/(fitted_value*(1-fitted_value)))


}


nu<-apply(nu,2,mean)

omega<-expected_i_covariate(optim_result$par,X,dim_x)/n*-1

 each_z<-NULL
 sigma<-NULL
 mu<-NULL
nu_c<-NULL
 for(j in 1:dim(threshold_pair)[1]){
 
if(j==dim(threshold_pair)[1]){ 
   
element<-sum(response_resid^2)

f<-fitted_value*(1-fitted_value)

fx<-cbind(f,X_general)
nu<-NULL
for(i in 1:dim(fx)[2]){

nu<-cbind(nu,fx[,i]*(1-2*fitted_value)/(fitted_value*(1-fitted_value)))


}

 
 nu<-apply(nu,2,sum)/n
 
nu_c<-rbind(nu_c,nu)

sigma_square<-mean((1-2*fitted_value)^2/(fitted_value*(1-fitted_value)))-t(nu)%*%solve(omega)%*%nu

 sigma<-c(sigma,sigma_square)

}

if(j!=dim(threshold_pair)[1]){

nc<-sum(risk>=threshold_pair[j,2])+sum(risk<=threshold_pair[j,1])

element<-sum(response_resid[risk>=threshold_pair[j,2]]^2)+sum(response_resid[risk<=threshold_pair[j,1]]^2)
 
 
 f<-fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]]*(1-fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]])
 
fx<-cbind(f,X_general[(risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]),])

nu<-NULL
for(i in 1:dim(fx)[2]){

nu<-cbind(nu,fx[,i]*(1-2*fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]])/f)

}

 #nu<-(1-2*fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]])/f*fx

 nu<-apply(nu,2,sum)/n
 
nu_c<-rbind(nu_c,nu)

  
 sigma_square<-sum((1-2*fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]])^2/(fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]]*(1-fitted_value[risk>=threshold_pair[j,2]|risk<=threshold_pair[j,1]])))/n-t(nu)%*%solve(omega)%*%nu
  
 sigma<-c(sigma,sigma_square)

 }
 }


pairwise<-apply(t(combn(1:dim(threshold_pair)[1],2)),1,pairwise_covariance,threshold_pair,nu_c,risk,fitted_value,omega)


 csigma<-c(sigma,pairwise)

no_order<-cbind(rbind(matrix(rep(1:dim(threshold_pair)[1],2),ncol=2),t(combn(1:dim(threshold_pair)[1],2))),csigma )


intermediate_sigma<-no_order[order(no_order[,1],no_order[,2]),]

Sigma<-xpnd(intermediate_sigma[,3],dim(threshold_pair)[1])

Sigma<-cov2cor(Sigma)

 
 simulated_sample<-mvrnorm(n=n_simulations,rep(0,dim(threshold_pair)[1]),Sigma)
 
 empirical_max<-apply(abs(simulated_sample),1,max)
 
marginal_test_stat<-test_statistic/sqrt(csigma[1:length(test_statistic)])


current_max<-max(abs(marginal_test_stat))
 
 marginal_p<-NULL

for(j in 1:length(marginal_test_stat)){

marginal_p<-c(marginal_p,2*(1-pnorm(abs(marginal_test_stat[j]))))
}

pval<-length(empirical_max[empirical_max>=current_max])/length(empirical_max)

hl<-hosmerlem(Y,fitted_value,g=10)


retval<-list(coefficients=c(optim_result$par[1],optim_result$par[2:(dim_x+1)]-1,optim_result$par[-(1:(dim_x+1))]),fitted.values=fitted_value, cov=solve(omega*n), TBM_ts=current_max, TBM_pv=pval, Windmeijer_ts=abs(marginal_test_stat[length(marginal_test_stat)]), Windmeijer_pv=marginal_p[length(marginal_p)], HL_ts=hl$chisq, HL_pv=hl$p.value)





return(retval)

}


